program ExampleCallback;

{$ifdef MSWINDOWS}{$apptype CONSOLE}{$endif}
{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

uses
  SysUtils, IPConnection, BrickletColor;

type
  TExample = class
  private
    ipcon: TIPConnection;
    c: TBrickletColor;
  public
    procedure ColorCB(sender: TBrickletColor;
                      const r: word; const g: word; const b: word; const c_: word);
    procedure Execute;
  end;

const
  HOST = 'localhost';
  PORT = 4223;
  UID = 'XYZ'; { Change to your UID }

var
  e: TExample;

{ Callback procedure for color callback }
procedure TExample.ColorCB(sender: TBrickletColor;
                           const r: word; const g: word; const b: word; const c_: word);
begin
  WriteLn(Format('Color[R]: %d', [r]));
  WriteLn(Format('Color[G]: %d', [g]));
  WriteLn(Format('Color[B]: %d', [b]));
  WriteLn(Format('Color[C]: %d', [c_]));
  WriteLn('');
end;

procedure TExample.Execute;
begin
  { Create IP connection }
  ipcon := TIPConnection.Create;

  { Create device object }
  c := TBrickletColor.Create(UID, ipcon);

  { Connect to brickd }
  ipcon.Connect(HOST, PORT);
  { Don't use device before ipcon is connected }

  { Register color callback to procedure ColorCB }
  c.OnColor := {$ifdef FPC}@{$endif}ColorCB;

  { Set period for color callback to 1s (1000ms)
    Note: The color callback is only called every second
          if the color has changed since the last call! }
  c.SetColorCallbackPeriod(1000);

  WriteLn('Press key to exit');
  ReadLn;
  ipcon.Destroy; { Calls ipcon.Disconnect internally }
end;

begin
  e := TExample.Create;
  e.Execute;
  e.Destroy;
end.
