{
  This file was automatically generated on 2016-02-10.

  Delphi/Lazarus Bindings Version 2.1.10

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletRealTimeClock;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_REAL_TIME_CLOCK_DEVICE_IDENTIFIER = 268;
  BRICKLET_REAL_TIME_CLOCK_DEVICE_DISPLAY_NAME = 'Real-Time Clock Bricklet';

  BRICKLET_REAL_TIME_CLOCK_FUNCTION_SET_DATE_TIME = 1;
  BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_DATE_TIME = 2;
  BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_TIMESTAMP = 3;
  BRICKLET_REAL_TIME_CLOCK_FUNCTION_SET_OFFSET = 4;
  BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_OFFSET = 5;
  BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_IDENTITY = 255;


  BRICKLET_REAL_TIME_CLOCK_WEEKDAY_MONDAY = 1;
  BRICKLET_REAL_TIME_CLOCK_WEEKDAY_TUESDAY = 2;
  BRICKLET_REAL_TIME_CLOCK_WEEKDAY_WEDNESDAY = 3;
  BRICKLET_REAL_TIME_CLOCK_WEEKDAY_THURSDAY = 4;
  BRICKLET_REAL_TIME_CLOCK_WEEKDAY_FRIDAY = 5;
  BRICKLET_REAL_TIME_CLOCK_WEEKDAY_SATURDAY = 6;
  BRICKLET_REAL_TIME_CLOCK_WEEKDAY_SUNDAY = 7;

type
  TArray0To2OfUInt8 = array [0..2] of byte;

  /// <summary>
  ///  Battery-backed real-time clock
  /// </summary>
  TBrickletRealTimeClock = class(TDevice)
  private
  protected
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid__: string; ipcon_: TIPConnection);

    /// <summary>
    ///  Sets the current date (including weekday) and the current time with hundredths
    ///  of a second resolution.
    ///  
    ///  Possible value ranges:
    ///  
    ///  * Year: 2000 to 2099
    ///  * Month: 1 to 12 (January to December)
    ///  * Day: 1 to 31
    ///  * Hour: 0 to 23
    ///  * Minute: 0 to 59
    ///  * Second: 0 to 59
    ///  * Centisecond: 0 to 99
    ///  * Weekday: 1 to 7 (Monday to Sunday)
    ///  
    ///  If the backup battery is installed then the real-time clock keeps date and
    ///  time even if the Bricklet is not powered by a Brick.
    ///  
    ///  The real-time clock handles leap year and inserts the 29th of February
    ///  accordingly. But leap seconds, time zones and daylight saving time are not
    ///  handled.
    /// </summary>
    procedure SetDateTime(const year: word; const month: byte; const day: byte; const hour: byte; const minute: byte; const second: byte; const centisecond: byte; const weekday: byte); virtual;

    /// <summary>
    ///  Returns the current date (including weekday) and the current time of the
    ///  real-time clock with hundredths of a second resolution.
    /// </summary>
    procedure GetDateTime(out year: word; out month: byte; out day: byte; out hour: byte; out minute: byte; out second: byte; out centisecond: byte; out weekday: byte); virtual;

    /// <summary>
    ///  Returns the current date and the time of the real-time clock converted to
    ///  milliseconds. The timestamp has an effective resolution of hundredths of a
    ///  second.
    /// </summary>
    function GetTimestamp: int64; virtual;

    /// <summary>
    ///  Sets the offset the real-time clock should compensate for in 2.17 ppm steps
    ///  between -277.76 ppm (-128) and +275.59 ppm (127).
    ///  
    ///  The real-time clock time can deviate from the actual time due to the frequency
    ///  deviation of its 32.768 kHz crystal. Even without compensation (factory
    ///  default) the resulting time deviation should be at most ±20 ppm (±52.6
    ///  seconds per month).
    ///  
    ///  This deviation can be calculated by comparing the same duration measured by the
    ///  real-time clock (``rtc_duration``) an accurate reference clock
    ///  (``ref_duration``).
    ///  
    ///  For best results the configured offset should be set to 0 ppm first and then a
    ///  duration of at least 6 hours should be measured.
    ///  
    ///  The new offset (``new_offset``) can be calculated from the currently configured
    ///  offset (``current_offset``) and the measured durations as follow::
    ///  
    ///    new_offset = current_offset - round(1000000 * (rtc_duration - ref_duration) / rtc_duration / 2.17)
    ///  
    ///  If you want to calculate the offset, then we recommend using the calibration
    ///  dialog in Brick Viewer, instead of doing it manually.
    ///  
    ///  The offset is saved in the EEPROM of the Bricklet and only needs to be
    ///  configured once.
    /// </summary>
    procedure SetOffset(const offset: shortint); virtual;

    /// <summary>
    ///  Returns the offset as set by <see cref="BrickletRealTimeClock.TBrickletRealTimeClock.SetOffset"/>.
    /// </summary>
    function GetOffset: shortint; virtual;

    /// <summary>
    ///  Returns the UID, the UID where the Bricklet is connected to, 
    ///  the position, the hardware and firmware version as well as the
    ///  device identifier.
    ///  
    ///  The position can be 'a', 'b', 'c' or 'd'.
    ///  
    ///  The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
    ///  |device_identifier_constant|
    /// </summary>
    procedure GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word); override;
  end;

implementation

constructor TBrickletRealTimeClock.Create(const uid__: string; ipcon_: TIPConnection);
begin
  inherited Create(uid__, ipcon_);
  apiVersion[0] := 2;
  apiVersion[1] := 0;
  apiVersion[2] := 0;

  responseExpected[BRICKLET_REAL_TIME_CLOCK_FUNCTION_SET_DATE_TIME] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_DATE_TIME] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_TIMESTAMP] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_REAL_TIME_CLOCK_FUNCTION_SET_OFFSET] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_OFFSET] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_IDENTITY] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;

end;

procedure TBrickletRealTimeClock.SetDateTime(const year: word; const month: byte; const day: byte; const hour: byte; const minute: byte; const second: byte; const centisecond: byte; const weekday: byte);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REAL_TIME_CLOCK_FUNCTION_SET_DATE_TIME, 17);
  LEConvertUInt16To(year, 8, request);
  LEConvertUInt8To(month, 10, request);
  LEConvertUInt8To(day, 11, request);
  LEConvertUInt8To(hour, 12, request);
  LEConvertUInt8To(minute, 13, request);
  LEConvertUInt8To(second, 14, request);
  LEConvertUInt8To(centisecond, 15, request);
  LEConvertUInt8To(weekday, 16, request);
  SendRequest(request);
end;

procedure TBrickletRealTimeClock.GetDateTime(out year: word; out month: byte; out day: byte; out hour: byte; out minute: byte; out second: byte; out centisecond: byte; out weekday: byte);
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_DATE_TIME, 8);
  response := SendRequest(request);
  year := LEConvertUInt16From(8, response);
  month := LEConvertUInt8From(10, response);
  day := LEConvertUInt8From(11, response);
  hour := LEConvertUInt8From(12, response);
  minute := LEConvertUInt8From(13, response);
  second := LEConvertUInt8From(14, response);
  centisecond := LEConvertUInt8From(15, response);
  weekday := LEConvertUInt8From(16, response);
end;

function TBrickletRealTimeClock.GetTimestamp: int64;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_TIMESTAMP, 8);
  response := SendRequest(request);
  result := LEConvertInt64From(8, response);
end;

procedure TBrickletRealTimeClock.SetOffset(const offset: shortint);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REAL_TIME_CLOCK_FUNCTION_SET_OFFSET, 9);
  LEConvertInt8To(offset, 8, request);
  SendRequest(request);
end;

function TBrickletRealTimeClock.GetOffset: shortint;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_OFFSET, 8);
  response := SendRequest(request);
  result := LEConvertInt8From(8, response);
end;

procedure TBrickletRealTimeClock.GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word);
var request, response: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REAL_TIME_CLOCK_FUNCTION_GET_IDENTITY, 8);
  response := SendRequest(request);
  uid := LEConvertStringFrom(8, 8, response);
  connectedUid := LEConvertStringFrom(16, 8, response);
  position := LEConvertCharFrom(24, response);
  for i := 0 to 2 do hardwareVersion[i] := LEConvertUInt8From(25 + (i * 1), response);
  for i := 0 to 2 do firmwareVersion[i] := LEConvertUInt8From(28 + (i * 1), response);
  deviceIdentifier := LEConvertUInt16From(31, response);
end;

end.
